﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Workflow.Runtime;
using System.Workflow.Runtime.Hosting;
using System.Workflow.ComponentModel;
using System.Workflow.ComponentModel.Serialization;
using System.Runtime.Serialization;

namespace SimpleWorkflows
{
    class Program
    {
        static void Main(string[] args)
        {
            using(WorkflowRuntime workflowRuntime = new WorkflowRuntime())
            {
                SqlWorkflowPersistenceService sql =
                    new SqlWorkflowPersistenceService(
                        "server=.;database=workflowpersistence;integrated security=SSPI",
                        true, new TimeSpan(0, 2, 0), new TimeSpan(0, 0, 10));
                workflowRuntime.AddService(sql);

                workflowRuntime.StartRuntime();

                WorkflowInstance instance = null;

                AutoResetEvent waitHandle = new AutoResetEvent(false);
                workflowRuntime.WorkflowCompleted += delegate(object sender, WorkflowCompletedEventArgs e) {waitHandle.Set();};
                workflowRuntime.WorkflowTerminated += delegate(object sender, WorkflowTerminatedEventArgs e)
                {
                    Console.ForegroundColor = ConsoleColor.Yellow;
                    Console.WriteLine(e.Exception.Message);
                    Console.ResetColor();
                    waitHandle.Set();
                };
                workflowRuntime.WorkflowIdled += delegate(object sender, WorkflowEventArgs wie)
                {
                    //waitHandle.Set();
                    ThreadPool.QueueUserWorkItem(new WaitCallback(delegate(object state)
                    {
                        WorkflowChanges changes = new WorkflowChanges(instance.GetWorkflowDefinition());
                        CompositeActivity root = changes.TransientWorkflow;
                        WriteLineActivity wl = new WriteLineActivity
                        {
                            Name = "newWriteLine",
                            OutputText = "dynamically added"
                        };
                        root.Activities.Add(wl);
                        instance.ApplyWorkflowChanges(changes);
                           
                    }));
                };
                workflowRuntime.ServicesExceptionNotHandled += delegate(object sender, ServicesExceptionNotHandledEventArgs snh)
                {
                    Console.ForegroundColor = ConsoleColor.Yellow;
                    Console.WriteLine(snh.Exception.Message);
                    Console.ResetColor();
                };
                
                instance = workflowRuntime.CreateWorkflow(typeof(SimpleWorkflows.Workflow1));
                instance.Start();

                waitHandle.WaitOne();
                Console.ReadLine();

            }
        }
    }
}
